@@ -6,6 +6,7 @@ from pathlib import Path
 from typing import Any
 from typing import Callable
 from typing import Hashable
+from typing import Iterable
 from typing import Optional
 from typing import Tuple
 from typing import TypeVar
@@ -31,16 +32,19 @@ from pandas import Index
 from pandas import Series
 from pandas.testing import assert_index_equal
 
+from joblib_windower.errors import DistinctIndicesError
+from joblib_windower.slide_ndarrays import are_equal_indices
+from joblib_windower.slide_ndarrays import Arguments
+from joblib_windower.slide_ndarrays import CPU_COUNT
+from joblib_windower.slide_ndarrays import datetime64ns
+from joblib_windower.slide_ndarrays import DEFAULT_STR_LEN_FACTOR
+from joblib_windower.slide_ndarrays import is_not_none
+from joblib_windower.slide_ndarrays import NaT
+from joblib_windower.slide_ndarrays import pandas_obj_to_ndarray
 from joblib_windower.slide_ndarrays import slide_ndarrays
-from joblib_windower.utilities import Arguments
-from joblib_windower.utilities import CPU_COUNT
-from joblib_windower.utilities import datetime64ns
-from joblib_windower.utilities import DEFAULT_STR_LEN_FACTOR
-from joblib_windower.utilities import is_not_none
-from joblib_windower.utilities import NaT
-from joblib_windower.utilities import pandas_obj_to_ndarray
-from joblib_windower.utilities import TEMP_DIR
-from joblib_windower.utilities import timedelta64ns
+from joblib_windower.slide_ndarrays import TEMP_DIR
+from joblib_windower.slide_ndarrays import timedelta64ns
+
 
 T = TypeVar("T")
 
@@ -68,10 +72,14 @@ class SeriesMetadata:
     index_name: Hashable
 
 
-def call_with_packing(keys: CTuple[Optional[str]], func: Callable[..., T], *args: Any) -> T:
+def call_with_packing(
+    keys: CTuple[Optional[str]], func: Callable[..., T], *args: Any,
+) -> T:
     packed = CTuple(args).grouper(3).map(pack_argument)
     is_kwargs, is_args = keys.zip(packed).partition(lambda x: x[0] is None)
-    arguments = Arguments(args=is_args.map(itemgetter(1)), kwargs=is_kwargs.dict())
+    arguments = Arguments(
+        args=is_args.map(itemgetter(1)), kwargs=is_kwargs.dict(),
+    )
     return func(*arguments.args, **arguments.kwargs)
 
 
@@ -117,7 +125,12 @@ def get_maybe_unique_dataframe_columns(arguments: Arguments) -> Optional[Index]:
 
 
 def get_maybe_unique_series_name(arguments: Arguments) -> Optional[Hashable]:
-    names = arguments.map_values(get_maybe_series_name).all_values().filter(is_not_none).set()
+    names = (
+        arguments.map_values(get_maybe_series_name)
+        .all_values()
+        .filter(is_not_none)
+        .set()
+    )
     try:
         return names.one()
     except (EmptyIterableError, MultipleElementsError):
@@ -125,20 +138,37 @@ def get_maybe_unique_series_name(arguments: Arguments) -> Optional[Hashable]:
 
 
 def get_maybe_unique_ndframe_index(arguments: Arguments) -> Optional[Index]:
-    indices = arguments.map_values(get_maybe_ndframe_index).all_values().filter(is_not_none)
+    indices = (
+        arguments.map_values(get_maybe_ndframe_index)
+        .all_values()
+        .filter(is_not_none)
+    )
     if indices:
-        try:
-            for index1, index2 in indices.combinations(2):
-                assert_index_equal(index1, index2)
-        except AssertionError:
-            return None
-        else:
-            index, *_ = indices
-            return index
+        return get_unique_index(*indices)
     else:
         raise ValueError("Expected at least 1 Series or DataFrame; got none")
 
 
+def get_unique_index(indices: Iterable[Index]) -> Index:
+    indices = CList(indices)
+    pairs = indices.combinations(2)
+    if pairs.starmap(
+        lambda x, y: are_equal_indices(x, y, check_names=True),
+    ).all():
+        index, *_ = indices
+        return index
+    else:
+        unequal, equal = pairs.partition(
+            lambda x: are_equal_indices(x[0], x[1], check_names=False),
+        )
+        if unequal:
+            (x, y), *_ = unequal
+            raise DistinctIndicesError(x, y)
+        else:
+            index, *_ = indices
+            return index.rename(None)
+
+
 def get_ndframe_spec(x: dtype) -> NDFrameSpec:
     if CIterable([bool_, str_]).map(lambda y: issubdtype(x, y)).any():
         return NDFrameSpec(dtype=dtype(object), masked=nan)
@@ -151,19 +181,24 @@ def get_ndframe_spec(x: dtype) -> NDFrameSpec:
 
 
 def masked_array_to_pandas_object(
-    array: MaskedArray, index: Index, name: Optional[Hashable], columns: Optional[Index],
+    array: MaskedArray,
+    index: Index,
+    name: Optional[Hashable],
+    columns: Optional[Index],
 ) -> Union[Series, DataFrame]:
     spec = get_ndframe_spec(array.dtype)
     if array.ndim == 1:
-        return Series(data=array.data, index=index, dtype=spec.dtype, name=name).where(
-            ~array.mask, spec.masked,
-        )
+        return Series(
+            data=array.data, index=index, dtype=spec.dtype, name=name,
+        ).where(~array.mask, spec.masked)
     elif array.ndim == 2:
-        m, n = array.shape
+        _, n = array.shape
         return DataFrame(
             data=array.data,
-            index=index if len(index) == m else None,
-            columns=columns if columns is not None and len(columns) == n else None,
+            index=index,
+            columns=columns
+            if columns is not None and len(columns) == n
+            else None,
         ).where(~array.mask, spec.masked)
     elif array.ndim == 3:
         length, *_ = array.shape
@@ -179,7 +214,9 @@ def masked_array_to_pandas_object(
 
 def pack_argument(
     data: Tuple[
-        Optional[Union[IndexMetadata, SeriesMetadata, DataFrameMetadata]], Any, Optional[ndarray],
+        Optional[Union[IndexMetadata, SeriesMetadata, DataFrameMetadata]],
+        Any,
+        Optional[ndarray],
     ],
 ) -> Any:
     metadata, x, maybe_index = data
@@ -188,14 +225,18 @@ def pack_argument(
     elif isinstance(metadata, SeriesMetadata):
         if isinstance(maybe_index, ndarray):
             return Series(
-                x, index=Index(maybe_index, name=metadata.index_name), name=metadata.name,
+                x,
+                index=Index(maybe_index, name=metadata.index_name),
+                name=metadata.name,
             )
         else:
             return x
     elif isinstance(metadata, DataFrameMetadata):
         if isinstance(maybe_index, ndarray):
             return DataFrame(
-                x, index=Index(maybe_index, name=metadata.index_name), columns=metadata.columns,
+                x,
+                index=Index(maybe_index, name=metadata.index_name),
+                columns=metadata.columns,
             )
         else:
             return Series(x, index=metadata.columns, name=maybe_index)
@@ -206,9 +247,10 @@ def pack_argument(
 def slide_ndframes(
     func: Callable,
     *args: Any,
-    window: int = 1,
+    window: Optional[int] = None,
+    lag: Optional[int] = None,
+    step: Optional[int] = None,
     min_frac: Optional[float] = None,
-    step: int = 1,
     temp_dir: Union[Path, str] = TEMP_DIR,
     str_len_factor: int = DEFAULT_STR_LEN_FACTOR,
     parallel: bool = False,
@@ -225,14 +267,17 @@ def slide_ndframes(
         partial(call_with_packing, keys, func),
         *unpacked_arguments.flatten(),
         window=window,
-        min_frac=min_frac,
+        lag=lag,
         step=step,
+        min_frac=min_frac,
         temp_dir=temp_dir,
         str_len_factor=str_len_factor,
         parallel=parallel,
         processes=processes,
     )
-    return masked_array_to_pandas_object(masked_array, index, maybe_name, maybe_columns)
+    return masked_array_to_pandas_object(
+        masked_array, index, maybe_name, maybe_columns,
+    )
 
 
 def unpack_argument(x: Any) -> Tuple[Any, Any, Optional[ndarray]]:
@@ -247,7 +292,9 @@ def unpack_argument(x: Any) -> Tuple[Any, Any, Optional[ndarray]]:
         return None, x, None
 
 
-def unpack_dataframe(x: DataFrame) -> Tuple[DataFrameMetadata, ndarray, ndarray]:
+def unpack_dataframe(
+    x: DataFrame,
+) -> Tuple[DataFrameMetadata, ndarray, ndarray]:
     return (
         DataFrameMetadata(index_name=x.index.name, columns=x.columns),
         pandas_obj_to_ndarray(x),
